//
//  PhobosDevice.m
//  GMPhobos
//
//  Created by Q14 on 2020/9/22.
//

#import "PhobosDevice.h"
#import <AdSupport/ASIdentifierManager.h>
#import <SAMKeychain/SAMKeychain.h>
#import <GMCache/GMCache.h>
#import "PhobosUtil.h"

NSString *const KprivatePolicyNotification = @"gm_popup_private_policy_notification";
//钥匙串账户
NSString *const keyChainAccount = @"com.wanmeizhensuo.ZhengXing.phobos.keychain";
//钥匙串账户 表示device_id
NSString *const DEVICEID = @"device_id";
//钥匙串账户 表示idfa
NSString *const IDFA = @"idfa";
// 默认获取不到的idfa
NSString *defaultIDFA = @"00000000-0000-0000-0000-000000000000";


static dispatch_queue_t queue;

@implementation PhobosDevice

#pragma mark - getIDFA

/*同步获取IDFA 直接从钥匙串中获取 适配ATTrackingManagerAuthorization可使用下面方法
+ (NSString *)getIDFA {
    if (![PhobosDevice showPrivatePolicy]) {
        return defaultIDFA;
    }
    //直接从钥匙串中获取
    NSString *keyChainIDFA = [[ASIdentifierManager sharedManager] advertisingIdentifier].UUIDString;
    if ([self checkIdfaIsNull:keyChainIDFA]) {
        //如果为空从钥匙串中取获取
        keyChainIDFA = [self getKeyChainService:IDFA account:keyChainAccount];
    } else {
        //如果获取到存储到钥匙串中
        [self saveKeyChainWithValue:keyChainIDFA service:IDFA account:keyChainAccount];
    }
    if ([self checkIdfaIsNull:keyChainIDFA]) {
        keyChainIDFA = defaultIDFA;
    }
    return keyChainIDFA;
}*/

/** iOS 14 适配 ATTrackingManagerAuthorizationStatus */
 //同步获取IDFA 直接从钥匙串中获取
+ (NSString *)getIDFA {
    if (![PhobosDevice showPrivatePolicy]) {
        return defaultIDFA;
    }
    //直接从钥匙串中获取
    NSString *keyChainIDFA = [self getKeyChainService:IDFA account:keyChainAccount];
    if ([self checkIdfaIsNull:keyChainIDFA]) {
        keyChainIDFA = defaultIDFA;
    }
    return keyChainIDFA;
}

/// 请求获取IDFA授权
/// @param completeBlock 回调执行结果
+ (void)requestTrackingAuthorizaWithCompletionHandler:(PhobosGetIDFACompleteBlock)completeBlock {
    
    // iOS14及以上版本需要先请求权限
    if (@available(iOS 14, *)) {
        [ATTrackingManager requestTrackingAuthorizationWithCompletionHandler:^(ATTrackingManagerAuthorizationStatus status) {
            dispatch_async(dispatch_get_main_queue(), ^{
                NSString *idfaString;
                if (status == ATTrackingManagerAuthorizationStatusAuthorized) {
                    idfaString = [[ASIdentifierManager sharedManager] advertisingIdentifier].UUIDString;
                    if (![self checkIdfaIsNull:idfaString]) {
                        [self saveKeyChainWithValue:idfaString service:IDFA account:keyChainAccount];
                    }
                } else {
                    // NSLog(@"请在设置-隐私-跟踪中允许App请求跟踪");
                }
                if (completeBlock) {
                    completeBlock(idfaString,status);
                }
            });
        }];
    } else {
        // iOS14以下版本依然使用原方式访问IDFA
        // 判断在设置-隐私里用户是否打开了广告跟踪
        NSInteger status = 0;
        NSString *idfaString;
        if ([[ASIdentifierManager sharedManager] isAdvertisingTrackingEnabled]) {
            idfaString = [[[ASIdentifierManager sharedManager] advertisingIdentifier] UUIDString];
            if (![self checkIdfaIsNull:idfaString]) {
                // 标示 用户已授权打开
                status = 3;
                [self saveKeyChainWithValue:idfaString service:IDFA account:keyChainAccount];
            }
        } else {
            // 表示用户拒绝打开
            status = 2;
            // NSLog(@"请在设置-隐私-广告中打开广告跟踪功能");
        }
        if (completeBlock) {
            completeBlock(idfaString,status);
        }
    }
}

#pragma mark - 对IDFA判空处理
//检查idfa 是否为空
+ (BOOL)checkIdfaIsNull:(NSString *)idfa {
    return !idfa.length || [idfa isEqualToString:defaultIDFA];
}

#pragma mark - getIDFV
+ (NSString *)getIDFV {
    if (![PhobosDevice showPrivatePolicy]) {
        return defaultIDFA;
    }
    return [[[UIDevice currentDevice] identifierForVendor] UUIDString];
}

#pragma mark - getDEVICEID
+ (NSString *)getDEVICEID {
    if (![PhobosDevice showPrivatePolicy]) {
        return defaultIDFA;
    }
    NSString *idfa = [self getIDFA];
    if ([self checkIdfaIsNull:idfa]) {
        idfa = [self getUQID];
    }
    return idfa;
}

+ (NSString *)getUQID {
    if (![PhobosDevice showPrivatePolicy]) {
        return defaultIDFA;
    }
    NSString *deviceId = [self getKeyChainService:DEVICEID account:keyChainAccount];
    if ([self checkIdfaIsNull:deviceId]) {
        deviceId = [self getIDFV];
        [self saveKeyChainWithValue:deviceId service:DEVICEID account:keyChainAccount];
    }
    return deviceId;
}


#pragma mark - 钥匙串封装
+ (void)saveKeyChainWithValue:(NSString *)value service:(NSString *)service account:(NSString *)account {
    //避免在重复相同的key 重复设置
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        queue = dispatch_queue_create("com.device.idfa.serial.queue", DISPATCH_QUEUE_SERIAL);
    });
    
    if (queue && [PhobosUtil isNonEmpty:service] && ![self checkIdfaIsNull:value]) {
        dispatch_sync(queue, ^{
            [SAMKeychain setPassword:value forService:service account:account];
            [SAMKeychain setAccessibilityType:kSecAttrAccessibleAlwaysThisDeviceOnly];
        });
    }
}

//根据 service 与账户keyChain 获取存储的值
+ (NSString *)getKeyChainService:(NSString *)service account:(NSString *)account {
    return [SAMKeychain passwordForService:service account:account];
}

+ (BOOL)showPrivatePolicy {
    return [GMCache fetchObjectAtDocumentPathWithkey:KprivatePolicyNotification];
}

@end
